# Holtrop.py
# master thesis
# Author: Stian Rakke
# Spring 2016
import math
import sqlite3 as lite
import sys
from Ship import Ship
import numpy as np


class Holtrop(object):
    def __init__(self, ship, speeds):
        self._ship = ship

        self._speeds = speeds
        self.shipspeed = None  # AIS
        self.gravity = 9.802  # Greenwich
        self.rho = 1025.0
        self.kinviscosity = 0.00000118831

    def getBrakePowers(self):
        brakepower = {}
        for speed in self._speeds:
            self.shipspeed = speed[0] / 10.0
            brakepower[speed[0] / 10.0] = self.brakePower()

        return brakepower

    #########  Functions Holtrop #########

    def calcc1(self):
        breadth = self._ship.breadth
        draughtforep = self._ship.draught
        ie = self.calcie()
        c1 = 2223105.0 * self.calcc7() ** (3.78613) * (draughtforep / breadth) ** (1.07961) * (90.0 - ie) ** (-1.37566)
        return (c1)

    def calcc2(self):
        c2 = math.exp(-1.89 * math.sqrt(self.calcc3()))
        return (c2)

    def calcc3(self):
        transbulb = self._ship.transbulbarea
        breadth = self._ship.breadth
        draughtforep = self._ship.draught
        centerofbulb = self._ship.cobkeel

        c3 = 0.56 * transbulb ** 1.5 / \
             (breadth * draughtforep * (0.31 * math.sqrt(transbulb) + draughtforep - centerofbulb))
        return (c3)

    def calcc4(self):
        lengthwl = self._ship.lengthwl
        draughtforep = self._ship.draught
        if draughtforep / lengthwl <= 0.04:
            c4 = draughtforep / lengthwl
        else:
            c4 = 0.04
        return c4

    def calcc5(self):
        transom = self._ship.transom
        breadth = self._ship.breadth
        midshipC = self._ship.cm
        draughtforep = self._ship.draught
        c5 = 1.0 - 0.8 * transom / (breadth * draughtforep * midshipC)
        return c5

    def calcc6(self):
        if self.calcfnt() < 5.0:
            c6 = 0.2 * (1.0 - 0.2 * self.calcfnt())
        else:
            c6 = 0.0
        return c6

    def calcc7(self):
        breadth = self._ship.breadth
        lengthwl = self._ship.lengthwl

        if (breadth / lengthwl) < 0.11:
            c7 = 0.229577 * (breadth / lengthwl) ** 0.33333
        elif (breadth / lengthwl) > 0.11 and (breadth / lengthwl) < 0.25:
            c7 = (breadth / lengthwl)
        else:
            c7 = 0.5 - 0.625 * (breadth / lengthwl)
        return c7

    def calcc8(self):
        breadth = self._ship.breadth
        lengthwl = self._ship.lengthwl
        draughtaftp = self._ship.draught
        propdiameter = self._ship.propdiameter
        if breadth / draughtaftp < 5.0:
            c8 = breadth * self.calcs() / (lengthwl * propdiameter * draughtaftp)
        else:
            c8 = self.calcs() * (7.0 * breadth / draughtaftp - 25.0) / (
            lengthwl * propdiameter * (breadth / draughtaftp - 3.0))
        return c8

    def calcc9(self):
        if self.calcc8() < 28.0:
            c9 = self.calcc8()
        else:
            c9 = 32.0 - 16.0 / (self.calcc8() - 24.0)
        return c9

    def calcc10(self):
        breadth = self._ship.breadth
        lengthwl = self._ship.lengthwl
        if lengthwl / breadth > 5.0:
            c10 = breadth / lengthwl
        else:
            c10 = 0.25 - 0.003328402 / (breadth / lengthwl - 0.134615385)
        return c10

    def calcc11(self):
        breadth = self._ship.breadth
        draughtaftp = self._ship.draught
        propdiameter = self._ship.propdiameter
        if draughtaftp / propdiameter < 2.0:
            c11 = draughtaftp / propdiameter
        else:
            c11 = 0.0833333 * (draughtaftp / propdiameter) ** 3.0 + 1.33333
        return c11

    def calcc12(self):
        breadth = self._ship.breadth
        lengthwl = self._ship.lengthwl
        draughtforep = self._ship.draught
        propdiameter = self._ship.propdiameter
        if draughtforep / lengthwl > 0.05:
            c12 = (draughtforep / lengthwl) ** 0.2228446
        elif draughtforep / lengthwl > 0.02 and draughtforep / lengthwl < 0.05:
            c12 = 48.2 * (draughtforep / lengthwl - 0.02) ** 2.078 + 0.479948
        else:
            c12 = 0.479948
        return c12

    def calcc13(self):
        c13 = 1.0 + 0.003 * self.calcsternc()
        return c13

    def calcc15(self):
        disp = self._ship.displacement
        lengthwl = self._ship.lengthwl
        if lengthwl ** 3 / disp < 512:
            c15 = -1.69385
        elif lengthwl ** 3 / disp > 1727.0:
            c15 = 0
        else:
            c15 = -1.69385 + (lengthwl / disp ** (1.0 / 3.0) - 8.0 / 2.36)
        return c15

    def calcc16(self):
        cp = self._ship.cp
        if cp < 0.8:
            c16 = 8.07981 * cp - 13.8673 * cp ** 2.0 + 6.984388 * cp ** 3
        else:
            c16 = 1.73014 - 0.7067 * cp
        return c16

    def calcp0minpv(self):
        p0minpv = 99047.0
        return p0minpv

    def calcrgh(self):
        rgh = self.rho * self.gravity * self._ship.cobkeel
        return rgh

    def calcshipspeed(self):
        speed = self.shipspeed * 0.5144
        return speed

    def calcs(self):
        lengthwl = self._ship.lengthwl
        draughtforep = self._ship.draught
        breadth = self._ship.breadth
        midshipC = self._ship.cm
        cb = self._ship.cb
        cwp = self._ship.cwp
        transbulb = self._ship.transbulbarea
        s = lengthwl * (2 * draughtforep + breadth) * math.sqrt(midshipC) * (
        0.453 + 0.4425 * cb - 0.2862 * midshipC - 0.003467 * (
        breadth / draughtforep) + 0.3696 * cwp) + 2.38 * transbulb / cb
        return s

    def calcsapp(self):
        sapp = self._ship.sapp
        return sapp

    def calcsternc(self):
        sternCo = self._ship.sternparameter
        return sternCo

    def calcdisp(self):
        disp = self._ship.displacement
        return disp

    def calccp(self):
        cp = self._ship.cp
        return cp

    def calccp1(self):
        cp1 = 1.45 * self.calccp() - 0.315 - 0.0225 * self.calclcb()
        return cp1

    def calcm1(self):
        lengthwl = self._ship.lengthwl
        breadth = self._ship.breadth
        draughtforep = self._ship.draught
        disp = self._ship.displacement
        m1 = 0.0140407 * lengthwl / draughtforep - 1.75254 * disp ** (
        1.0 / 3.0) / lengthwl - 4.79323 * breadth / lengthwl - self.calcc16()
        return m1

    def calcm2(self):
        cp = self._ship.cp
        m2 = self.calcc15() * cp ** 2.0 * math.exp(-0.1 * self.calcfn() ** -2.0)
        return m2

    def calcreyn(self):
        lengthwl = self._ship.lengthwl
        reyn = self.calcshipspeed() * lengthwl / self.kinviscosity
        return reyn

    def calclambda(self):
        lengthwl = self._ship.lengthwl
        breadth = self._ship.breadth
        cp = self._ship.cp
        if lengthwl / breadth < 12.0:
            lambda1 = 1.446 * cp - 0.03 * lengthwl / breadth
        else:
            lambda1 = 1.446 * cp - 0.36
        return lambda1

    def calcfn(self):
        lengthwl = self._ship.lengthwl
        fn = self.calcshipspeed() / math.sqrt(self.gravity * lengthwl)
        return fn

    def calcfni(self):
        draughtforep = self._ship.draught
        centerofbulb = self._ship.cobkeel
        transbulb = self._ship.transbulbarea

        fni = self.calcshipspeed() / (math.sqrt(self.gravity * (
        draughtforep - centerofbulb - 0.25 * math.sqrt(transbulb)) + 0.15 * self.calcshipspeed() ** 2))
        return fni

    def calcfnt(self):
        transomarea = self._ship.transom
        breadth = self._ship.breadth
        cwp = self._ship.cwp
        fnt = self.calcshipspeed() / math.sqrt(2 * self.gravity * transomarea / (breadth + breadth * cwp))
        return fnt

    def calcca(self):
        lengthwl = self._ship.lengthwl
        cb = self._ship.cb
        ca = 0.006 * (lengthwl + 100.0) ** -0.16 - 0.00205 + 0.003 * math.sqrt(
            lengthwl / 7.5) * cb ** 4 * self.calcc1() * (0.04 - self.calcc4())
        return ca

    def calccf(self):
        cf = 0.075 / (math.log10(self.calcreyn()) - 2.0) ** 2.0
        return cf

    def calcct(self):
        ct = 0
        return ct

    def calccb(self):
        cb = self._ship.cb
        return cb

    def calccv(self):
        cv = self.calcformfactor() * self.calccf() + self.calcca()
        return cv

    def calcie(self):
        breadth = self._ship.breadth
        lengthwl = self._ship.lengthwl
        cwp = self._ship.cwp
        cp = self._ship.cp
        lpp = self._ship.lpp
        lcb = -0.75 * (lpp / 2.0) / 100.0
        lr = lengthwl * (1.0 - cp + ((0.06 * cp * lcb) / (4.0 * cp - 1.0)))
        displacement = self._ship.displacement
        ie = 1.0 + 89.0 * math.exp(
            -(lengthwl / breadth) ** 0.80856 * (1.0 - cwp) ** 0.30484 * (1.0 - cp - 0.0225 * lcb) ** 0.6367 * (
            lr / breadth) ** 0.34574 * (100.0 * displacement / lengthwl ** 3.0) ** 0.16302)
        return ie

    def calclcb(self):
        lpp = self._ship.lpp
        lcb = -0.75 * (lpp / 2) / 100
        return lcb

    def calclengthofrun(self):
        lengthwl = self._ship.lengthwl
        cp = self._ship.cp
        cb = self._ship.cb
        lcb = self.calclcb()
        lr = lengthwl * (1.0 - cp + ((0.06 * cp * lcb) / (4.0 * cp - 1.0)))
        return lr

    def calcpitch(self):
        pitch = self._ship.pitch
        return pitch

    def calcd(self):
        # from Holtrop approximation sheet
        d = -0.9
        return d

    def calcformfactor(self):
        breadth = self._ship.breadth
        cp = self._ship.cp
        formfactor = self.calcc13() * (
        0.93 + self.calcc12() * ((breadth / self.calclengthofrun()) ** 0.92497) * ((0.95 - cp) ** -0.521448) * (
        (1.0 - cp + 0.0225 * self.calclcb()) ** 0.6906))
        return formfactor

    def calcformfactor2(self, sapprudderbehindskeg, sapprudderbehindstern, sapptwinscrewbalancerudder,
                        sappshaftbrackets, sappskeg, sappstrutbossings, sapphullbossings, sappshafts,
                        sappstabilazerfins, sappdome, sappbilgekeels):
        return 1.50

    def calct(self, a=''):
        lengthwl = self._ship.lengthwl
        breadth = self._ship.breadth
        cb = self._ship.cb
        propdiameter = self._ship.propdiameter
        draughtforep = self._ship.draught
        cp1 = self.calccp1()
        sternprop = self._ship.sternparameter
        if a is 'twinscrew':  # twinscrew
            t = 0.325 * cb - 0.1885 * propdiameter / math.sqrt(breadth * draughtforep)
        elif a is 'singlescrewopenstern':  # singlescrew, open sternon slender fast sailing ships:
            t = 0.1
        elif a is 'singlescrewconventionalstern':  # single screw, conventional stern
            t = 0.001979 * lengthwl / (
            breadth - breadth * cp1) + 1.0585 * self.calcc10() - 0.00524 - 0.1418 * propdiameter ** 2.0 / (
            breadth * draughtforep) + 0.0015 * sternprop
        else:
            t = 0.001979 * lengthwl / (
            breadth - breadth * cp1) + 1.0585 * self.calcc10() - 0.00524 - 0.1418 * propdiameter ** 2.0 / (
            breadth * draughtforep) + 0.0015 * sternprop
        return t

    def calcw(self, a=''):
        cb = self._ship.cb
        propdiameter = self._ship.propdiameter
        breadth = self._ship.breadth
        draughtforep = self._ship.draught
        lengthwl = self._ship.lengthwl
        cp1 = self.calccp1()
        cp = self._ship.cp
        sternc = self._ship.sternparameter
        if a is 'twinscrew':  # twinscrew :#twinscrew
            w = 0.3095 * cb + 10.0 * self.calccv() * cb - 0.23 * propdiameter / math.sqrt(breadth * draughtforep)
        elif a is 'singlescrewopenstern':  # singlescrew, open sternon slender fast sailing ships:
            w = 0.3 * cb + 10.0 * self.calccv() * cb - 0.1
        elif a is 'singlescrewconventionalstern':  # single screw, conventional stern
            w = self.calcc9() * self.calccv() * lengthwl / draughtforep * (
            0.0661875 + 1.21756 * self.calcc11() * self.calccv() / (1.0 - cp1)) + 0.24558 * math.sqrt(
                breadth / (lengthwl * (1.0 - cp1))) - 0.09726 / (0.95 - cp) + 0.11434 / (
            0.95 - cb) + 0.75 * sternc * self.calccv() + 0.002 * sternc
        else:
            w = self.calcc9() * self.calccv() * lengthwl / draughtforep * (
            0.0661875 + 1.21756 * self.calcc11() * self.calccv() / (1.0 - cp1)) + 0.24558 * math.sqrt(
                breadth / (lengthwl * (1.0 - cp1))) - 0.09726 / (0.95 - cp) + 0.11434 / (
            0.95 - cb) + 0.75 * sternc * self.calccv() + 0.002 * sternc
        return w

    def calclrdll(self):
        cp = self._ship.cp
        cb = self._ship.cb
        lrdll = 1.0 - cp + 0.06 * cp * cb / (4.0 * cp - 1.0)
        return lrdll

    def calcdeltacd(self):
        delcd = (2.0 + 4.0 * self.calctdlc075()) * (
        0.003605 - (1.89 + 1.62 * math.log10(self.calcc075() / self.calckp())) ** -2.5)
        return delcd

    def calctdlc075(self):
        nopropblades = self._ship.numberOfPropellerBlades
        propdiameter = self._ship.propdiameter
        tdlc075 = (0.0185 - 0.00125 * nopropblades) * propdiameter / self.calcc075()
        return tdlc075

    def calcc075(self):
        propdiameter = self._ship.propdiameter
        nopropblades = self._ship.numberOfPropellerBlades
        c075 = 2.073 * (self.calcaedlao()) * propdiameter / nopropblades
        return c075

    def calcaedlao(self):
        nopropblades = self._ship.numberOfPropellerBlades
        propdiameter = self._ship.propdiameter
        k = self.calck()
        aedlao1 = k + (1.3 + 0.3 * nopropblades) * (self.calcthrust() * 1000) / (
        propdiameter ** 2.0 * (self.calcp0minpv() + self.calcrgh()))
        return aedlao1

    def calcz(self):
        z = self._ship.numberOfPropellerBlades
        return z

    def calck(self):
        # k=0.1
        k = self._ship.calck()
        return k

    def calckp(self):
        # standard
        kp = 0.00003
        return kp

    def calckts(self):
        #
        kts = 0
        return kts

    def calckqs(self):
        #
        kqs = 0
        return kqs

    def calcnr(self):
        # From B-series polynomials
        nr = self._ship.nreff
        return nr

    def calcn0(self):
        # From B-series polynomials
        n0 = self._ship.noeff
        return n0

    def calcns(self):
        #
        ns = self._ship.noeff
        return ns

    def calcnh(self):
        nh = (1.0 - self.calct()) / (1.0 - self.calcw())
        return nh

    def calcnd(self):
        nd = self.calcnh() * self.calcn0() * self.calcnr()
        return nd

    def calcrf(self):
        rf = 0.5 * self.rho * self.calcshipspeed() ** 2.0 * self.calcs() * self.calccf()
        rf = rf / 1000.0
        return rf

    def calcrapp(self):
        rapp = 0.5 * self.rho * self.calcshipspeed() ** 2.0 * self.calcsapp() * (
        self.calcformfactor2(1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)) * self.calccf()
        rapp = rapp / 1000.0
        return rapp

    def calcrw(self):
        rw = self.calcc1() * self.calcc2() * self.calcc5() * self.calcdisp() * self.rho * self.gravity * math.exp(
            self.calcm1() * self.calcfn() ** (self.calcd()) + self.calcm2() * math.cos(
                self.calclambda() * self.calcfn() ** (-2.0)))
        rw = rw / 1000.0
        return rw

    def calcrb(self):
        transbulb = self._ship.transbulbarea
        rb = 0.11 * math.exp(-3.0 * self.calcpb() ** (-2.0)) * self.calcfni() ** 3.0 * transbulb ** (
        1.5) * self.rho * self.gravity / (1.0 + self.calcfni() ** 2.0)
        rb = rb / 1000.0
        return rb

    def calcrtr(self):
        transomearea = self._ship.transbulbarea
        rtr = 0.5 * self.rho * self.calcshipspeed() ** 2.0 * transomearea * self.calcc6()
        rtr = rtr / 1000.0
        return rtr

    def calcra(self):
        ra = 0.5 * self.rho * self.calcshipspeed() ** 2 * self.calcs() * self.calcca()
        ra = ra / 1000.0
        return ra

    def calcrt(self):
        rt = (
             self.calcrf() * self.calcformfactor()) + self.calcrapp() + self.calcrw() + self.calcrb() + self.calcrtr() + self.calcra()
        return rt

    def calcpe(self):
        pe = self.calcrt() * self.calcshipspeed()
        return pe

    def calcpb(self):
        transbulb = self._ship.transbulbarea
        draughtforep = self._ship.draught
        centerofbulb = self._ship.cobkeel
        pb = 0.56 * math.sqrt(transbulb) / (draughtforep - 1.5 * centerofbulb)
        return pb

    def calcthrust(self):
        thrust = self.calcpe() / self.calcshipspeed() * 1.0 / (1.0 - self.calct())
        return thrust

    def calcps(self):
        ps = self.calcpe() / (
        self.calcnr() * self.calcn0() * self.calcns() * (1.0 - self.calct()) / (1.0 - self.calcw()))
        return ps

    def calcpt(self):
        pt = self.calcpe() / self.calcnh()
        return pt

    def calcpd(self):
        # Arguments

        pd = self.calcpe() / self.calcnd()
        return pd

    def addseamargin(self):
        # Arguments
        seamargin = 0.85
        sm = self.calcpd() / seamargin
        return sm

    def add85MCRoperation(self):
        # Arguments
        mcr85 = 0.85
        mcr = self.addseamargin() / mcr85
        return mcr

    # Find ship brakepower
    def brakePower(self):
        ns = self._ship.nseff
        brakePower = self.addseamargin() * ns
        return brakePower
